#include <stdio.h>
#include <fcntl.h>
#include <limits.h>
#include <math.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include "generic.h"
#include "matrix.h"
#include "polygon.h"

#ifndef MAIN_H
#define MAIN_H

#define I_HAT     0
#define J_HAT     1
#define K_HAT     2
#define H_HAT     3

#define X         0
#define Y         1
#define Z         2
#define H         3

struct eye
{
   double View_direction[3];
   double Up[3];
   double Location[3];
};

typedef struct eye eye;

struct view
{
   int      Num_Eye_Points;
   int      Current_Eye_Point;
   eye     *Eye;
   double   Distance_to_Screen;
   window   Screen;

   double   Near_Plane_Distance;
   double   Convergence_Distance;
};

typedef struct view view;

struct world
{
   int      Num_Polygons;
   int      Max_Vertices_per_Polygon;
   polygon *Polygons;
   vertex  *Vertices;
};

typedef struct world world;

#endif

double **Get_Transformation_Matrix(void);

view  *Get_View_Info(void);
eye   *Get_Eye_Info(int Num_Eye_Points);
world *Get_World_Info(int Num_Polygons, int Num_Vertices);

void   Set_View_Transformation_Matrix(view *View,
                                      double **View_Transformation_Matrix);
void   Set_Image_Transformation_Matrix(window *Image, view *View,
                                       double **Image_Transformation_Matrix);

void   Transform_World(double **Transformation_Matrix, world *World);

